import logging

from sqlalchemy import Column, BigInteger
from sqlalchemy import Integer
from vortex.Tuple import Tuple, addTupleType

from peek_abstract_chunked_index.private.tuples.ACIProcessorQueueTupleABC import (
    ACIProcessorQueueTupleABC,
)
from peek_core_search._private.PluginNames import searchTuplePrefix
from .DeclarativeBase import DeclarativeBase

logger = logging.getLogger(__name__)


@addTupleType
class SearchIndexCompilerQueue(Tuple, DeclarativeBase, ACIProcessorQueueTupleABC):
    __tablename__ = "SearchIndexCompilerQueue"
    __tupleType__ = searchTuplePrefix + "SearchIndexCompilerQueueTable"

    id = Column(BigInteger, primary_key=True, autoincrement=True)
    chunkKey = Column(Integer, primary_key=True)

    @classmethod
    def sqlCoreLoad(cls, row):
        return SearchIndexCompilerQueue(id=row.id, chunkKey=row.chunkKey)

    @property
    def ckiUniqueKey(self):
        return self.chunkKey



"""initial tables

Peek Plugin Database Migration Script

Revision ID: 30f0d0f925ca
Revises: 
Create Date: 2018-06-09 20:19:50.662811

"""

# revision identifiers, used by Alembic.
revision = "30f0d0f925ca"
down_revision = None
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table(
        "EncodedSearchIndexChunk",
        sa.Column("id", sa.BigInteger(), nullable=False),
        sa.Column("chunkKey", sa.Integer(), nullable=False),
        sa.Column("encodedData", sa.LargeBinary(), nullable=False),
        sa.Column("encodedHash", sa.String(), nullable=False),
        sa.Column("lastUpdate", sa.String(), nullable=False),
        sa.PrimaryKeyConstraint("id", "chunkKey"),
        schema="core_search",
    )
    op.create_index(
        "idx_EncodedSearchIndex_chunkKey",
        "EncodedSearchIndexChunk",
        ["chunkKey"],
        unique=True,
        schema="core_search",
    )
    op.create_table(
        "EncodedSearchObjectChunk",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("chunkKey", sa.Integer(), nullable=False),
        sa.Column("encodedData", sa.LargeBinary(), nullable=False),
        sa.Column("encodedHash", sa.String(), nullable=False),
        sa.Column("lastUpdate", sa.String(), nullable=False),
        sa.PrimaryKeyConstraint("id", "chunkKey"),
        schema="core_search",
    )
    op.create_index(
        "idx_EncodedSearchObject_chunkKey",
        "EncodedSearchObjectChunk",
        ["chunkKey"],
        unique=True,
        schema="core_search",
    )
    op.create_table(
        "SearchIndexCompilerQueue",
        sa.Column("id", sa.Integer(), nullable=False, autoincrement=True),
        sa.Column("chunkKey", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("id", "chunkKey"),
        schema="core_search",
    )
    op.create_table(
        "SearchObjectCompilerQueue",
        sa.Column("id", sa.Integer(), nullable=False, autoincrement=True),
        sa.Column("chunkKey", sa.Integer(), nullable=False),
        sa.PrimaryKeyConstraint("id", "chunkKey"),
        schema="core_search",
    )
    op.create_table(
        "SearchObjectType",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("title", sa.String(), nullable=False),
        sa.PrimaryKeyConstraint("id"),
        schema="core_search",
    )
    op.create_index(
        "idx_ObjType_name",
        "SearchObjectType",
        ["name"],
        unique=True,
        schema="core_search",
    )
    op.create_index(
        "idx_ObjType_title",
        "SearchObjectType",
        ["title"],
        unique=True,
        schema="core_search",
    )
    op.create_table(
        "SearchProperty",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=False),
        sa.Column("title", sa.String(), nullable=False),
        sa.Column("order", sa.Integer(), server_default="0", nullable=False),
        sa.PrimaryKeyConstraint("id"),
        schema="core_search",
    )
    op.create_index(
        "idx_SearchProp_name",
        "SearchProperty",
        ["name"],
        unique=True,
        schema="core_search",
    )
    op.create_index(
        "idx_SearchProp_title",
        "SearchProperty",
        ["title"],
        unique=True,
        schema="core_search",
    )
    op.create_table(
        "Setting",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("name", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        schema="core_search",
    )
    op.create_table(
        "SearchObject",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("objectTypeId", sa.Integer(), nullable=False),
        sa.Column("key", sa.String(), nullable=False),
        sa.Column("chunkKey", sa.Integer(), nullable=False),
        sa.Column("propertiesJson", sa.String(), nullable=True),
        sa.Column("packedJson", sa.String(), nullable=True),
        sa.ForeignKeyConstraint(
            ["objectTypeId"], ["core_search.SearchObjectType.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_search",
    )
    op.create_index(
        "idx_SearchObject_chunkKey",
        "SearchObject",
        ["chunkKey"],
        unique=False,
        schema="core_search",
    )
    op.create_index(
        "idx_SearchObject_key",
        "SearchObject",
        ["key"],
        unique=True,
        schema="core_search",
    )
    op.create_index(
        "idx_SearchObject_objectTypeId",
        "SearchObject",
        ["objectTypeId"],
        unique=False,
        schema="core_search",
    )
    op.create_table(
        "SettingProperty",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("settingId", sa.Integer(), nullable=False),
        sa.Column("key", sa.String(length=50), nullable=False),
        sa.Column("type", sa.String(length=16), nullable=True),
        sa.Column("int_value", sa.Integer(), nullable=True),
        sa.Column("char_value", sa.String(), nullable=True),
        sa.Column("boolean_value", sa.Boolean(), nullable=True),
        sa.ForeignKeyConstraint(
            ["settingId"],
            ["core_search.Setting.id"],
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_search",
    )
    op.create_index(
        "idx_SettingProperty_settingId",
        "SettingProperty",
        ["settingId"],
        unique=False,
        schema="core_search",
    )
    op.create_table(
        "SearchIndex",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("chunkKey", sa.Integer(), nullable=False),
        sa.Column("keyword", sa.String(), nullable=False),
        sa.Column("propertyName", sa.String(), nullable=False),
        sa.Column("objectId", sa.Integer(), nullable=False),
        sa.ForeignKeyConstraint(
            ["objectId"], ["core_search.SearchObject.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_search",
    )
    op.create_index(
        "idx_SearchIndex_quick_query",
        "SearchIndex",
        ["chunkKey", "keyword", "propertyName", "objectId"],
        unique=True,
        schema="core_search",
    )
    op.create_table(
        "SearchObjectRoute",
        sa.Column("id", sa.Integer(), nullable=False),
        sa.Column("objectId", sa.Integer(), nullable=False),
        sa.Column("importGroupHash", sa.String(), nullable=False),
        sa.Column("routeTitle", sa.String(), nullable=False),
        sa.Column("routePath", sa.String(), nullable=False),
        sa.ForeignKeyConstraint(
            ["objectId"], ["core_search.SearchObject.id"], ondelete="CASCADE"
        ),
        sa.PrimaryKeyConstraint("id"),
        schema="core_search",
    )
    op.create_index(
        "idx_ObjectRoute_objectId",
        "SearchObjectRoute",
        ["objectId"],
        unique=False,
        schema="core_search",
    )
    op.create_index(
        "idx_ObjectRoute_routeTitle_importGroupHash",
        "SearchObjectRoute",
        ["importGroupHash"],
        unique=False,
        schema="core_search",
    )
    op.create_index(
        "idx_ObjectRoute_routeTitle_objectId",
        "SearchObjectRoute",
        ["routeTitle", "objectId"],
        unique=True,
        schema="core_search",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(
        "idx_ObjectRoute_routeTitle_objectId",
        table_name="SearchObjectRoute",
        schema="core_search",
    )
    op.drop_index(
        "idx_ObjectRoute_routeTitle_importGroupHash",
        table_name="SearchObjectRoute",
        schema="core_search",
    )
    op.drop_index(
        "idx_ObjectRoute_objectId", table_name="SearchObjectRoute", schema="core_search"
    )
    op.drop_table("SearchObjectRoute", schema="core_search")
    op.drop_index(
        "idx_SearchIndex_quick_query", table_name="SearchIndex", schema="core_search"
    )
    op.drop_table("SearchIndex", schema="core_search")
    op.drop_index(
        "idx_SettingProperty_settingId",
        table_name="SettingProperty",
        schema="core_search",
    )
    op.drop_table("SettingProperty", schema="core_search")
    op.drop_index(
        "idx_SearchObject_objectTypeId", table_name="SearchObject", schema="core_search"
    )
    op.drop_index(
        "idx_SearchObject_key", table_name="SearchObject", schema="core_search"
    )
    op.drop_index(
        "idx_SearchObject_chunkKey", table_name="SearchObject", schema="core_search"
    )
    op.drop_table("SearchObject", schema="core_search")
    op.drop_table("Setting", schema="core_search")
    op.drop_index(
        "idx_SearchProp_title", table_name="SearchProperty", schema="core_search"
    )
    op.drop_index(
        "idx_SearchProp_name", table_name="SearchProperty", schema="core_search"
    )
    op.drop_table("SearchProperty", schema="core_search")
    op.drop_index(
        "idx_SearchProp_title", table_name="SearchObjectType", schema="core_search"
    )
    op.drop_index(
        "idx_SearchProp_name", table_name="SearchObjectType", schema="core_search"
    )
    op.drop_table("SearchObjectType", schema="core_search")
    op.drop_table("SearchObjectCompilerQueue", schema="core_search")
    op.drop_table("SearchIndexCompilerQueue", schema="core_search")
    op.drop_index(
        "idx_EncodedSearchObject_chunkKey",
        table_name="EncodedSearchObjectChunk",
        schema="core_search",
    )
    op.drop_table("EncodedSearchObjectChunk", schema="core_search")
    op.drop_index(
        "idx_EncodedSearchIndex_chunkKey",
        table_name="EncodedSearchIndexChunk",
        schema="core_search",
    )
    op.drop_table("EncodedSearchIndexChunk", schema="core_search")
    # ### end Alembic commands ###



import logging
from typing import Dict

from peek_abstract_chunked_index.private.client.handlers.ACICacheHandlerABC import (
    ACICacheHandlerABC,
)
from peek_abstract_chunked_index.private.tuples.ACIUpdateDateTupleABC import (
    ACIUpdateDateTupleABC,
)
from peek_core_search._private.PluginNames import searchFilt
from peek_core_search._private.tuples.search_index.SearchIndexUpdateDateTuple import (
    SearchIndexUpdateDateTuple,
)

logger = logging.getLogger(__name__)

clientSearchIndexWatchUpdateFromDeviceFilt = {
    "key": "clientSearchIndexWatchUpdateFromDevice"
}
clientSearchIndexWatchUpdateFromDeviceFilt.update(searchFilt)


# ModelSet HANDLER
class SearchIndexCacheHandler(ACICacheHandlerABC):
    _UpdateDateTuple: ACIUpdateDateTupleABC = SearchIndexUpdateDateTuple
    _updateFromServerFilt: Dict = clientSearchIndexWatchUpdateFromDeviceFilt
    _logger: logging.Logger = logger



import logging
from typing import Dict

from peek_abstract_chunked_index.private.server.client_handlers.ACIChunkUpdateHandlerABC import (
    ACIChunkUpdateHandlerABC,
)
from peek_abstract_chunked_index.private.tuples.ACIEncodedChunkTupleABC import (
    ACIEncodedChunkTupleABC,
)
from peek_core_search._private.client.controller.SearchObjectCacheController import (
    clientSearchObjectUpdateFromServerFilt,
)
from peek_core_search._private.storage.EncodedSearchObjectChunk import (
    EncodedSearchObjectChunk,
)

logger = logging.getLogger(__name__)


class ClientSearchObjectChunkUpdateHandler(ACIChunkUpdateHandlerABC):
    _ChunkedTuple: ACIEncodedChunkTupleABC = EncodedSearchObjectChunk
    _updateFromServerFilt: Dict = clientSearchObjectUpdateFromServerFilt
    _logger: logging.Logger = logger



from typing import Any

from sqlalchemy import Column, LargeBinary, BigInteger
from sqlalchemy import Integer, String
from sqlalchemy.sql.schema import Index
from vortex.Tuple import Tuple, addTupleType

from peek_abstract_chunked_index.private.tuples.ACIEncodedChunkTupleABC import (
    ACIEncodedChunkTupleABC,
)
from peek_core_search._private.PluginNames import searchTuplePrefix
from peek_core_search._private.storage.DeclarativeBase import DeclarativeBase


@addTupleType
class EncodedSearchIndexChunk(Tuple, DeclarativeBase, ACIEncodedChunkTupleABC):
    __tablename__ = "EncodedSearchIndexChunk"
    __tupleType__ = searchTuplePrefix + "EncodedSearchIndexChunkTable"

    ENCODED_DATA_KEYWORD_NUM = 0
    ENCODED_DATA_PROPERTY_MAME_NUM = 1
    ENCODED_DATA_OBJECT_IDS_JSON_INDEX = 2

    id = Column(BigInteger, primary_key=True, autoincrement=True)

    chunkKey = Column(Integer, primary_key=True)
    encodedData = Column(LargeBinary, nullable=False)
    encodedHash = Column(String, nullable=False)
    lastUpdate = Column(String, nullable=False)

    __table_args__ = (Index("idx_EncodedSearchIndex_chunkKey", chunkKey, unique=True),)

    @property
    def ckiChunkKey(self):
        return self.chunkKey

    @property
    def ckiHasEncodedData(self) -> bool:
        return bool(self.encodedData)

    @property
    def ckiLastUpdate(self):
        return self.lastUpdate

    @classmethod
    def ckiCreateDeleteEncodedChunk(cls, chunkKey: Any):
        return EncodedSearchIndexChunk(chunkKey=chunkKey)

    @classmethod
    def sqlCoreChunkKeyColumn(cls):
        return cls.__table__.c.chunkKey

    @classmethod
    def sqlCoreLoad(cls, row):
        return EncodedSearchIndexChunk(
            id=row.id,
            chunkKey=row.chunkKey,
            encodedData=row.encodedData,
            encodedHash=row.encodedHash,
            lastUpdate=row.lastUpdate,
        )



from twisted.trial import unittest

from peek_core_search._private.worker.tasks.KeywordSplitter import (
    splitPartialKeywords,
    splitFullKeywords,
    _splitFullTokens,
    twoCharTokens,
)


class KeywordSplitterTest(unittest.TestCase):
    def testDuplicates(self):
        self.assertEqual({"smith"}, _splitFullTokens("smith smith"))

    def test_twoCharTokens(self):
        self.assertEqual(set(), twoCharTokens("smith smith"))

        self.assertEqual({"^to$"}, twoCharTokens(splitFullKeywords("two to")))

        self.assertEqual({"^to"}, twoCharTokens(splitPartialKeywords("two to")))

    def testFullKeywordSplit(self):
        self.assertEqual({"^smith$"}, splitFullKeywords("smith"))
        self.assertEqual({"^zorroreyner$"}, splitFullKeywords("ZORRO-REYNER"))
        self.assertEqual({"^34534535$"}, splitFullKeywords("34534535"))

        self.assertEqual({"^and$"}, splitFullKeywords("and"))
        self.assertEqual({"^to$"}, splitFullKeywords("to"))
        self.assertEqual({"^to$"}, splitFullKeywords("to"))

        self.assertEqual(
            {"^milton$", "^unit$", "^22$"}, splitFullKeywords("Milton Unit 22")
        )

        self.assertEqual({"^unit$"}, splitFullKeywords("Unit A"))

        self.assertEqual({"^unit$"}, splitFullKeywords("Unit 1"))

        self.assertEqual(
            {"^trans$", "^66kv$", "^b3$", "^cb$", "^ats$"},
            splitFullKeywords("ATS B3 TRANS 66KV CB"),
        )

    def testPartialKeywordSplit(self):
        self.assertEqual({"^smi", "mit", "ith"}, splitPartialKeywords("smith"))

        self.assertEqual(
            {"^zor", "orr", "rro", "ror", "ore", "rey", "eyn", "yne", "ner"},
            splitPartialKeywords("ZORRO-REYNER"),
        )
        self.assertEqual(
            {"^345", "535", "453", "534", "345"}, splitPartialKeywords("34534535")
        )

        self.assertEqual({"^and"}, splitPartialKeywords("and"))

        self.assertEqual({"^to"}, splitPartialKeywords("to"))

        self.assertEqual({"a55", "^ha5"}, splitPartialKeywords("ha55"))

        self.assertEqual(
            {"^mil", "ilt", "lto", "ton", "^uni", "nit", "^22"},
            splitPartialKeywords("Milton Unit 22"),
        )

        self.assertEqual(
            {"^mil", "ill", "lls", "^un", "^no"}, splitPartialKeywords("mills un no")
        )

        self.assertEqual({"^uni", "nit", "^22"}, splitPartialKeywords("Unit 22"))

        self.assertEqual({"^uni", "nit"}, splitPartialKeywords("Unit 1"))

        self.assertEqual({"^uni", "nit"}, splitPartialKeywords("A Unit"))

        self.assertEqual({"^uni", "nit"}, splitPartialKeywords("2 Unit"))

        self.assertEqual(
            {"^ats", "^cb", "^b3", "^66k", "6kv", "^tra", "ran", "ans"},
            splitPartialKeywords("ATS B3 TRANS 66KV CB"),
        )

        self.assertEqual({"^col", "^lin", "ins"}, splitPartialKeywords("COL LINS"))

        self.assertNotEqual(
            splitPartialKeywords("COLLINS"), splitPartialKeywords("COL LINS")
        )



from peek_core_search._private.client.controller.FastKeywordController import (
    FastKeywordController,
)
from peek_core_search._private.tuples.KeywordAutoCompleteTupleAction import (
    KeywordAutoCompleteTupleAction,
)
from peek_plugin_base.PeekVortexUtil import peekServerName
from peek_core_search._private.PluginNames import searchFilt
from peek_core_search._private.PluginNames import searchActionProcessorName
from vortex.handler.TupleActionProcessorProxy import TupleActionProcessorProxy


def makeTupleActionProcessorProxy(fastKeywordController: FastKeywordController):
    proxy = TupleActionProcessorProxy(
        tupleActionProcessorName=searchActionProcessorName,
        proxyToVortexName=peekServerName,
        additionalFilt=searchFilt,
    )

    proxy.setDelegate(KeywordAutoCompleteTupleAction.tupleType(), fastKeywordController)

    return proxy
